==> mlpack_3.4.2.orig.tar.xz: mlpack-3.4.2/doc/examples/sample-ml-app/sample-ml-app/sample-ml-app.cpp <==
/ * * 
 
   *   @ f i l e   s a m p l e - m l - a p p . c p p 
 
   *   @ a u t h o r   G e r m a n   L a n c i o n i 
 
   * 
 
   *   S a m p l e   a p p   b a s e d   o n   t h e   W i n d o w s   M L   A p p   T u t o r i a l . 
 
   *   Q u i c k l y   s h o w s   h o w   t o   c r e a t e   a   m a c h i n e   l e a r n i n g   a p p   u s i n g   m l p a c k / C + + . 
 
   * 
 
   *   m l p a c k   i s   f r e e   s o f t w a r e ;   y o u   m a y   r e d i s t r i b u t e   i t   a n d / o r   m o d i f y   i t   u n d e r   t h e 
 
   *   t e r m s   o f   t h e   3 - c l a u s e   B S D   l i c e n s e .     Y o u   s h o u l d   h a v e   r e c e i v e d   a   c o p y   o f   t h e 
 
   *   3 - c l a u s e   B S D   l i c e n s e   a l o n g   w i t h   m l p a c k .     I f   n o t ,   s e e 
 
   *   h t t p : / / w w w . o p e n s o u r c e . o r g / l i c e n s e s / B S D - 3 - C l a u s e   f o r   m o r e   i n f o r m a t i o n . 
 
   * / 
 
 # i n c l u d e   " s t d a f x . h " 
 
 
 
 u s i n g   n a m e s p a c e   a r m a ; 
 
 u s i n g   n a m e s p a c e   m l p a c k ; 
 
 u s i n g   n a m e s p a c e   m l p a c k : : t r e e ; 
 
 u s i n g   n a m e s p a c e   m l p a c k : : c v ; 
 
 
 
 / * 
 
 T h i s   s a m p l e   a p p   c o v e r s   a   b r i e f   e n d - t o - e n d   M L   w o r k f l o w 
 
 a s   y o u   w o u l d   d o   i n   a   r e a l - l i f e   a p p l i c a t i o n .   I n c l u d i n g : 
 
 -   L o a d i n g   a n d   p r e p a r i n g   a   d a t a s e t 
 
 -   T r a i n i n g   ( R a n d o m   F o r e s t   a s   e x a m p l e ) 
 
 -   C o m p u t i n g   t h e   t r a i n i n g   a c c u r a c y 
 
 -   C r o s s - V a l i d a t i o n   u s i n g   K - F o l d 
 
 -   M e t r i c s   g a t h e r i n g   ( a c c u r a c y ,   p r e c i s i o n ,   r e c a l l ,   F 1 ) 
 
 -   S a v i n g   t h e   t r a i n e d   m o d e l 
 
 -   L o a d i n g   t h e   m o d e l 
 
 -   C l a s s i f y i n g   a   n e w   s a m p l e 
 
 
 
 A s s u m p t i o n s : 
 
 -   N o   l a b e l s   n o r m a l i z a t i o n   r e q u i r e d 
 
 * / 
 
 i n t   m a i n ( ) 
 
 { 
 
 	 c o u t   < <   " [ S A M P L E : B E G I N ] " ; 
 
 
 
 	 / /   ( 1 )   L o a d   t h e   d a t a s e t 
 
 	 c o u t   < <   " \ n L o a d i n g   d a t a s e t . . . " ; 
 
 
 
 	 m a t   d a t a s e t ; 
 
 
 
 	 / /   C S V   i s   l o a d e d   t r a n s p o s e d   ( c o l u m n s   a r e   s a m p l e s ,   r o w s   a r e   d i m e n s i o n s ) 
 
 	 b o o l   l o a d e d   =   m l p a c k : : d a t a : : L o a d ( " d a t a / g e r m a n . c s v " ,   d a t a s e t ) ; 
 
 	 i f   ( ! l o a d e d ) 
 
 	 	 r e t u r n   - 1 ; 
 
 
 
 	 R o w < s i z e _ t >   l a b e l s ; 
 
 
 
 	 / /   E x t r a c t   t h e   l a b e l s   f r o m   t h e   l a s t   d i m e n s i o n   o f   t h e   t r a i n i n g   s e t 
 
 	 l a b e l s   =   c o n v _ t o < R o w < s i z e _ t > > : : f r o m ( d a t a s e t . r o w ( d a t a s e t . n _ r o w s   -   1 ) ) ; 
 
 
 
 	 / /   R e m o v e   t h e   l a b e l s   f r o m   t h e   t r a i n i n g   s e t 
 
 	 d a t a s e t . s h e d _ r o w ( d a t a s e t . n _ r o w s   -   1 ) ; 
 
 
 
 	 / /   ( 2 )   T r a i n i n g 
 
 	 c o u t   < <   " \ n T r a i n i n g . . . " ; 
 
 	 c o n s t   s i z e _ t   n u m C l a s s e s   =   2 ; 
 
 	 c o n s t   s i z e _ t   m i n i m u m L e a f S i z e   =   5 ; 
 
 	 c o n s t   s i z e _ t   n u m T r e e s   =   1 0 ; 
 
 
 
 	 R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t >   r f ; 
 
 
 
 	 r f   =   R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t > ( d a t a s e t ,   l a b e l s , 
 
 	 	 n u m C l a s s e s ,   n u m T r e e s ,   m i n i m u m L e a f S i z e ) ; 
 
 
 
 	 R o w < s i z e _ t >   p r e d i c t i o n s ; 
 
 	 r f . C l a s s i f y ( d a t a s e t ,   p r e d i c t i o n s ) ; 
 
 
 
 	 c o n s t   s i z e _ t   c o r r e c t   =   a r m a : : a c c u ( p r e d i c t i o n s   = =   l a b e l s ) ; 
 
 
 
 	 c o u t   < <   " \ n T r a i n i n g   A c c u r a c y :   "   < <   ( d o u b l e ( c o r r e c t )   /   d o u b l e ( l a b e l s . n _ e l e m ) ) ; 
 
 
 
 	 / /   ( 3 )   C r o s s - V a l i d a t i o n 
 
 	 c o u t   < <   " \ n C r o s s - V a l i d a t i n g . . . " ; 
 
 	 c o n s t   s i z e _ t   k   =   1 0 ; 
 
 
 
 	 K F o l d C V < R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t > ,   A c c u r a c y >   c v ( k , 
 
 	 	 d a t a s e t ,   l a b e l s ,   n u m C l a s s e s ) ; 
 
 
 
 	 d o u b l e   c v A c c   =   c v . E v a l u a t e ( n u m T r e e s ,   m i n i m u m L e a f S i z e ) ; 
 
 	 c o u t   < <   " \ n K F o l d C V   A c c u r a c y :   "   < <   c v A c c ; 
 
 
 
 	 d o u b l e   c v P r e c i s i o n   =   P r e c i s i o n < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n P r e c i s i o n :   "   < <   c v P r e c i s i o n ; 
 
 
 
 	 d o u b l e   c v R e c a l l   =   R e c a l l < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n R e c a l l :   "   < <   c v R e c a l l ; 
 
 
 
 	 d o u b l e   c v F 1   =   F 1 < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n F 1 :   "   < <   c v F 1 ; 
 
 
 
 	 / /   ( 4 )   S a v e   t h e   m o d e l 
 
 	 c o u t   < <   " \ n S a v i n g   m o d e l . . . " ; 
 
 	 m l p a c k : : d a t a : : S a v e ( " m y m o d e l . x m l " ,   " m o d e l " ,   r f ,   f a l s e ) ; 
 
 
 
 	 / /   ( 5 )   L o a d   t h e   m o d e l 
 
 	 c o u t   < <   " \ n L o a d i n g   m o d e l . . . " ; 
 
 	 m l p a c k : : d a t a : : L o a d ( " m y m o d e l . x m l " ,   " m o d e l " ,   r f ) ; 
 
 
 
 	 / /   ( 6 )   C l a s s i f y   a   n e w   s a m p l e 
 
 	 c o u t   < <   " \ n C l a s s i f y i n g   a   n e w   s a m p l e . . . " ; 
 
 	 / /   S h o u l d   c l a s s i f y   a s   " 1 " 
 
 	 m a t   s a m p l e ( " 2   1 2   2   1 3   1   2   2   1   3   2 4   3   1   1   1   1   1   0   1   0   1   0   0   0 " ) ; 
 
 
 
 	 m a t   p r o b a b i l i t i e s ; 
 
 
 
 	 r f . C l a s s i f y ( s a m p l e ,   p r e d i c t i o n s ,   p r o b a b i l i t i e s ) ; 
 
 
 
 	 u 6 4   r e s u l t   =   p r e d i c t i o n s . a t ( 0 ) ; 
 
 
 
 	 c o u t   < <   " \ n C l a s s i f i c a t i o n   r e s u l t :   "   < <   r e s u l t   < <   "   ,   P r o b a b i l i t i e s :   "   < < 
 
 	 	 p r o b a b i l i t i e s . a t ( 0 )   < <   " / "   < <   p r o b a b i l i t i e s . a t ( 1 ) ; 
 
 
 
 	 c o u t   < <   " \ n [ S A M P L E : E N D ] \ n " ; 
 
 	 r e t u r n   0 ; 
 
 } 
 
 
 
 

==> mlpack_4.6.0.orig.tar.xz: mlpack-4.6.0/doc/examples/sample-ml-app/sample-ml-app/sample-ml-app.cpp <==
/ * * 
 
   *   @ f i l e   s a m p l e - m l - a p p . c p p 
 
   *   @ a u t h o r   G e r m a n   L a n c i o n i 
 
   * 
 
   *   S a m p l e   a p p   b a s e d   o n   t h e   W i n d o w s   M L   A p p   T u t o r i a l . 
 
   *   Q u i c k l y   s h o w s   h o w   t o   c r e a t e   a   m a c h i n e   l e a r n i n g   a p p   u s i n g   m l p a c k / C + + . 
 
   * 
 
   *   m l p a c k   i s   f r e e   s o f t w a r e ;   y o u   m a y   r e d i s t r i b u t e   i t   a n d / o r   m o d i f y   i t   u n d e r   t h e 
 
   *   t e r m s   o f   t h e   3 - c l a u s e   B S D   l i c e n s e .     Y o u   s h o u l d   h a v e   r e c e i v e d   a   c o p y   o f   t h e 
 
   *   3 - c l a u s e   B S D   l i c e n s e   a l o n g   w i t h   m l p a c k .     I f   n o t ,   s e e 
 
   *   h t t p : / / w w w . o p e n s o u r c e . o r g / l i c e n s e s / B S D - 3 - C l a u s e   f o r   m o r e   i n f o r m a t i o n . 
 
   * / 
 
 # i n c l u d e   " s t d a f x . h " 
 
 
 
 u s i n g   n a m e s p a c e   a r m a ; 
 
 u s i n g   n a m e s p a c e   m l p a c k ; 
 
 u s i n g   n a m e s p a c e   s t d ; 
 
 
 
 / * 
 
 T h i s   s a m p l e   a p p   c o v e r s   a   b r i e f   e n d - t o - e n d   M L   w o r k f l o w 
 
 a s   y o u   w o u l d   d o   i n   a   r e a l - l i f e   a p p l i c a t i o n .   I n c l u d i n g : 
 
 -   L o a d i n g   a n d   p r e p a r i n g   a   d a t a s e t 
 
 -   T r a i n i n g   ( R a n d o m   F o r e s t   a s   e x a m p l e ) 
 
 -   C o m p u t i n g   t h e   t r a i n i n g   a c c u r a c y 
 
 -   C r o s s - V a l i d a t i o n   u s i n g   K - F o l d 
 
 -   M e t r i c s   g a t h e r i n g   ( a c c u r a c y ,   p r e c i s i o n ,   r e c a l l ,   F 1 ) 
 
 -   S a v i n g   t h e   t r a i n e d   m o d e l 
 
 -   L o a d i n g   t h e   m o d e l 
 
 -   C l a s s i f y i n g   a   n e w   s a m p l e 
 
 
 
 A s s u m p t i o n s : 
 
 -   N o   l a b e l s   n o r m a l i z a t i o n   r e q u i r e d 
 
 * / 
 
 i n t   m a i n ( ) 
 
 { 
 
 	 c o u t   < <   " [ S A M P L E : B E G I N ] " ; 
 
 
 
 	 / /   ( 1 )   L o a d   t h e   d a t a s e t 
 
 	 c o u t   < <   " \ n L o a d i n g   d a t a s e t . . . " ; 
 
 
 
 	 m a t   d a t a s e t ; 
 
 
 
 	 / /   C S V   i s   l o a d e d   t r a n s p o s e d   ( c o l u m n s   a r e   s a m p l e s ,   r o w s   a r e   d i m e n s i o n s ) 
 
 	 b o o l   l o a d e d   =   m l p a c k : : d a t a : : L o a d ( " d a t a / g e r m a n . c s v " ,   d a t a s e t ) ; 
 
 	 i f   ( ! l o a d e d ) 
 
 	 	 r e t u r n   - 1 ; 
 
 
 
 	 R o w < s i z e _ t >   l a b e l s ; 
 
 
 
 	 / /   E x t r a c t   t h e   l a b e l s   f r o m   t h e   l a s t   d i m e n s i o n   o f   t h e   t r a i n i n g   s e t 
 
 	 l a b e l s   =   c o n v _ t o < R o w < s i z e _ t > > : : f r o m ( d a t a s e t . r o w ( d a t a s e t . n _ r o w s   -   1 ) ) ; 
 
 
 
 	 / /   R e m o v e   t h e   l a b e l s   f r o m   t h e   t r a i n i n g   s e t 
 
 	 d a t a s e t . s h e d _ r o w ( d a t a s e t . n _ r o w s   -   1 ) ; 
 
 
 
 	 / /   ( 2 )   T r a i n i n g 
 
 	 c o u t   < <   " \ n T r a i n i n g . . . " ; 
 
 	 c o n s t   s i z e _ t   n u m C l a s s e s   =   2 ; 
 
 	 c o n s t   s i z e _ t   m i n i m u m L e a f S i z e   =   5 ; 
 
 	 c o n s t   s i z e _ t   n u m T r e e s   =   1 0 ; 
 
 
 
 	 R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t >   r f ; 
 
 
 
 	 r f   =   R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t > ( d a t a s e t ,   l a b e l s , 
 
 	 	 n u m C l a s s e s ,   n u m T r e e s ,   m i n i m u m L e a f S i z e ) ; 
 
 
 
 	 R o w < s i z e _ t >   p r e d i c t i o n s ; 
 
 	 r f . C l a s s i f y ( d a t a s e t ,   p r e d i c t i o n s ) ; 
 
 
 
 	 c o n s t   s i z e _ t   c o r r e c t   =   a r m a : : a c c u ( p r e d i c t i o n s   = =   l a b e l s ) ; 
 
 
 
 	 c o u t   < <   " \ n T r a i n i n g   A c c u r a c y :   "   < <   ( d o u b l e ( c o r r e c t )   /   d o u b l e ( l a b e l s . n _ e l e m ) ) ; 
 
 
 
 	 / /   ( 3 )   C r o s s - V a l i d a t i o n 
 
 	 c o u t   < <   " \ n C r o s s - V a l i d a t i n g . . . " ; 
 
 	 c o n s t   s i z e _ t   k   =   1 0 ; 
 
 
 
 	 K F o l d C V < R a n d o m F o r e s t < G i n i G a i n ,   R a n d o m D i m e n s i o n S e l e c t > ,   A c c u r a c y >   c v ( k , 
 
 	 	 d a t a s e t ,   l a b e l s ,   n u m C l a s s e s ) ; 
 
 
 
 	 d o u b l e   c v A c c   =   c v . E v a l u a t e ( n u m T r e e s ,   m i n i m u m L e a f S i z e ) ; 
 
 	 c o u t   < <   " \ n K F o l d C V   A c c u r a c y :   "   < <   c v A c c ; 
 
 
 
 	 d o u b l e   c v P r e c i s i o n   =   P r e c i s i o n < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n P r e c i s i o n :   "   < <   c v P r e c i s i o n ; 
 
 
 
 	 d o u b l e   c v R e c a l l   =   R e c a l l < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n R e c a l l :   "   < <   c v R e c a l l ; 
 
 
 
 	 d o u b l e   c v F 1   =   F 1 < B i n a r y > : : E v a l u a t e ( r f ,   d a t a s e t ,   l a b e l s ) ; 
 
 	 c o u t   < <   " \ n F 1 :   "   < <   c v F 1 ; 
 
 
 
 	 / /   ( 4 )   S a v e   t h e   m o d e l 
 
 	 c o u t   < <   " \ n S a v i n g   m o d e l . . . " ; 
 
 	 m l p a c k : : d a t a : : S a v e ( " m y m o d e l . x m l " ,   " m o d e l " ,   r f ,   f a l s e ) ; 
 
 
 
 	 / /   ( 5 )   L o a d   t h e   m o d e l 
 
 	 c o u t   < <   " \ n L o a d i n g   m o d e l . . . " ; 
 
 	 m l p a c k : : d a t a : : L o a d ( " m y m o d e l . x m l " ,   " m o d e l " ,   r f ) ; 
 
 
 
 	 / /   ( 6 )   C l a s s i f y   a   n e w   s a m p l e 
 
 	 c o u t   < <   " \ n C l a s s i f y i n g   a   n e w   s a m p l e . . . " ; 
 
 	 / /   S h o u l d   c l a s s i f y   a s   " 1 " 
 
 	 m a t   s a m p l e ( " 2   1 2   2   1 3   1   2   2   1   3   2 4   3   1   1   1   1   1   0   1   0   1   0   0   0 " ) ; 
 
 
 
 	 m a t   p r o b a b i l i t i e s ; 
 
 
 
 	 r f . C l a s s i f y ( s a m p l e ,   p r e d i c t i o n s ,   p r o b a b i l i t i e s ) ; 
 
 
 
 	 u 6 4   r e s u l t   =   p r e d i c t i o n s . a t ( 0 ) ; 
 
 
 
 	 c o u t   < <   " \ n C l a s s i f i c a t i o n   r e s u l t :   "   < <   r e s u l t   < <   "   ,   P r o b a b i l i t i e s :   "   < < 
 
 	 	 p r o b a b i l i t i e s . a t ( 0 )   < <   " / "   < <   p r o b a b i l i t i e s . a t ( 1 ) ; 
 
 
 
 	 c o u t   < <   " \ n [ S A M P L E : E N D ] \ n " ; 
 
 	 r e t u r n   0 ; 
 
 } 
 
 
 
 
